﻿using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;

namespace MAUIListEntry;

public partial class MainPage : ContentPage
{
	public MainPage()
	{
		InitializeComponent();
	}
    /// <summary>
    /// Handle leaving an entry field either because it lost focus or was marked completed
    /// </summary>
    /// <param name="ent">The entry field we are interested in</param>
    private void ProcessLeaveEntry(Entry ent)
    {
        if (!(ent.BindingContext is LineItem li)) return;
        if (ent == SelectedNameEntry)
            li.ItemName = ent.Text;
        else if (ent == SelectedAmountEntry)
        {
            if (ent.Text.Length > 9) // too many digits to format nicely
                return;
            if (string.IsNullOrWhiteSpace(ent.Text)) // nothing to parse
                return;
            if (Decimal.TryParse(ent.Text, out decimal d))
                li.Amount = d; // It's a legitimate value
        }
    }
    private void SelectedAmountEntry_Completed(object sender, EventArgs e)
    {

    }

    private void OnUnFocus(object sender, FocusEventArgs e)
    {
        if (sender is Entry ent)
            ProcessLeaveEntry(ent);
    }

    private void OnFocus(object sender, FocusEventArgs e)
    {

    }

    private void Button_Clicked(object sender, EventArgs e)
    {
        if (LineItemsListView.SelectedItem is LineItem li)
        {
            li.Amount = li.Amount * 10;
        }
    }
}

internal class LineItemView
{
    public ObservableCollection<LineItem> LineItems { get; } = new() {
        new LineItem() { ItemName = "Item 1",    Amount = 12},
        new LineItem() { ItemName = "Last Item", Amount = 1.42M } 
    };
}

internal class LineItem : INotifyPropertyChanged
{
    private string itemName;
    private string amountText;
    private decimal amount;

    public string ItemName { get => itemName; set => SetProperty(ref itemName, value); }
    public string AmountText { get => amountText; private set => SetProperty(ref amountText, value); }
    public decimal Amount 
    { 
        get => amount;
        set => SetProperty(ref amount, value, () => AmountText = String.Format("{0:C}", value));
    }

    #region Property change helper
    protected bool SetProperty<T>(ref T backingStore, T value,
        Action onChanged = null,
        [CallerMemberName] string propertyName = "")
    {
        if (EqualityComparer<T>.Default.Equals(backingStore, value))
            return false;

        backingStore = value;
        onChanged?.Invoke();
        OnPropertyChanged(propertyName);
        return true;
    }
    #endregion

    #region INotifyPropertyChanged
    public event PropertyChangedEventHandler PropertyChanged;
    protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = "")
    {
        PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
    }
    #endregion

}

